<?php

declare(strict_types=1);

namespace {{namespace}}\Hook;

use apexl\Io\hooks\AbstractHook;

final readonly class ConsoleCommands extends AbstractHook
{
    protected static function hooks(): array
    {
        return [];
    }

    public function __invoke(): void
    {
        /** @TODO Implement hook */
    }
}
