{
  "name": "{{name}}",
  "version": "0.0.1",
  "description": "",
  "main": "index.js",
  "scripts": {
    "test": "echo \"Error: no test specified\" && exit 1",
    "release": "npx release-it"
  },
  "repository": {
    "type": "git",
    "url": "git+ssh://git@bitbucket.org/N1ghteyes/{{name}}.git"
  },
  "author": "",
  "license": "MIT",
  "bugs": {
    "url": "https://bitbucket.org/N1ghteyes/io{{name}}/issues"
  },
  "homepage": "https://bitbucket.org/N1ghteyes/{{name}}#readme",
  "devDependencies": {
    "release-it": "^15.6.0"
  }
}
